/**********************************************************************
 *
 * formattools.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: formattools.cpp 442 1999-08-10 22:38:09Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.7  1999/08/10 22:38:08  sjboddie
   added some more format options

   Revision 1.6  1999/07/30 02:25:42  sjboddie
   made format_date function global

   Revision 1.5  1999/07/21 05:00:00  sjboddie
   added some date formatting

   Revision 1.4  1999/07/20 03:02:15  sjboddie
   added an [icon] option, added ability to call get_formatted_string
   with icon and link arguments set

   Revision 1.3  1999/07/09 02:44:35  sjboddie
   fixed parent(All) function so it only outputs parents and not current
   level meta

   Revision 1.2  1999/07/08 20:48:33  rjmcnab
   Added ability to print the result number

   Revision 1.1  1999/07/07 05:49:34  sjboddie
   had another crack at the format string code - created a new formattools
   module. It can now handle {If} and {Or} statements although there's a
   bug preventing nested if's and or's.

 */


#include "formattools.h"


// a few function prototypes
static text_t format_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
			     const text_t &link, const text_t &icon, const text_t &text);

static bool parse_action (text_t::const_iterator &here, const text_t::const_iterator &end, 
			  format_t *formatlistptr, text_tarray &metadata, bool &getParents,
			  text_tmap &metamap, int &metacount);

void metadata_t::clear() {
  metaname.clear();
  metaindex = 0;
  parentcommand = pNone;
  parentindex = 0;
  parentoptions.clear();
};

void decision_t::clear() {
  command = dMeta;
  meta.clear();
};

void format_t::clear() {
  command = comText;
  decision.clear();
  text.clear();
  meta.clear();
  nextptr = NULL;
  ifptr = NULL;
  elseptr = NULL;
  orptr = NULL;
};

void formatinfo_t::clear() {
  DocumentImages = false;
  // note that DocumentTopPages is used in receptionist when translating
  // OIDs. changing the default here may require changing it there too.
  DocumentTopPages = true;
  DocumentHeading = "{Or}{[parent(Top):Title],[Title],untitled}";
  DocumentArrowsTop = true;
  DocumentArrowsBottom = true;
  DocumentGoTo = true;
  DocumentButtons.erase (DocumentButtons.begin(), DocumentButtons.end());
  DocumentButtons.push_back ("Detach");
  DocumentButtons.push_back ("Highlight");
  DocumentButtons.push_back ("Expand Text");
  DocumentButtons.push_back ("Expand Contents");
  DocumentText = "[Text]";
}


// returns a date of form _textmonthnn_ 31, 1999 
// input is date of type 19991231
// at least the year must be present in date
text_t format_date (const text_t &date) {

  if (date.size() < 4) return "";

  text_t::const_iterator datebegin = date.begin();

  text_t year = substr (datebegin, datebegin+4);

  if (date.size() < 6) return year;

  text_t month = "_textmonth" + substr (datebegin+4, datebegin+6) + "_";
  int imonth = month.getint();
  if (imonth < 0 || imonth > 12) return year;

  if (date.size() < 8) return month + ", " + year;

  text_t day = substr (datebegin+6, datebegin+8);
  if (day[0] == '0') day = substr (day.begin()+1, day.end());
  int iday = day.getint();
  if (iday < 0 || iday > 31) return month + ", " + year;
    
  return month + " " + day + ", " + year;
}  

static void get_parent_options (text_t &instring, metadata_t &metaoption) {

  metaoption.clear();
  text_t meta, com, op;
  bool inbraces = false;
  bool inquotes = false;
  bool foundcolon = false;
  text_t::const_iterator here = instring.begin()+6;
  text_t::const_iterator end = instring.end();
  while (here != end) {
    if (*here == '(') inbraces = true;
    else if (*here == ')') inbraces = false;
    else if (*here == '\'' && !inquotes) inquotes = true;
    else if (*here == '\'' && inquotes) inquotes = false;
    else if (*here == ':' && !inbraces) foundcolon = true;
    else if (foundcolon) meta.push_back (*here);
    else if (inquotes) op.push_back (*here);
    else com.push_back (*here);
    here ++;
  }
  instring = meta;
  if (com.empty())
    metaoption.parentcommand = pImmediate;
  else if (com == "Top")
    metaoption.parentcommand = pTop;
  else if (is_number(com)) {
    metaoption.parentcommand = pIndex;
    metaoption.metaindex = com.getint();
  } else if (com == "All") {
    metaoption.parentcommand = pAll;
    metaoption.parentoptions = op;
  }
}

static bool parse_meta (text_t &meta, int &count, decision_t &decision,
			text_tarray &metadata, bool &getParents, text_tmap &metamap) {

  if (meta.size() > 7 && (substr (meta.begin(), meta.begin()+6) == "parent")) {
    getParents = true;
    metadata_t metaoption;
    get_parent_options (meta, metaoption);
    decision.meta = metaoption;
  }
  text_tmap::const_iterator it;
  if ((it = metamap.find(meta)) != metamap.end()) {
      decision.meta.metaindex = (*it).second.getint();
  } else {
    metamap[meta] = count;
    metadata.push_back (meta);
    decision.meta.metaindex = count;
    count ++;
  }
  return true;
}

static bool parse_meta (text_t &meta, int &count, format_t *formatlistptr,
			text_tarray &metadata, bool &getParents, text_tmap &metamap) {
  
  if (meta == "link")
    formatlistptr->command = comLink;
  else if (meta == "/link")
    formatlistptr->command = comEndLink;

  else if (meta == "num")
    formatlistptr->command = comNum;

  else if (meta == "icon")
    formatlistptr->command = comIcon;

  else if (meta == "Text")
    formatlistptr->command = comDoc;

  else {
    formatlistptr->command = comMeta;
    if (meta.size() > 7 && (substr (meta.begin(), meta.begin()+6) == "parent")) {
      getParents = true;
      metadata_t metaoption;
      get_parent_options (meta, metaoption);
      formatlistptr->meta = metaoption;
    }
    text_tmap::const_iterator it;
    if ((it = metamap.find(meta)) != metamap.end()) {
      formatlistptr->meta.metaindex = (*it).second.getint();
    } else {
      metamap[meta] = count;
      metadata.push_back (meta);
      formatlistptr->meta.metaindex = count;
      count ++;
    }
    formatlistptr->meta.metaname = meta;
  }
  return true;
}

static bool parse_string (const text_t &formatstring, format_t *formatlistptr, 
			  text_tarray &metadata, bool &getParents, 
			  text_tmap &metamap, int &metacount) {

  text_t text;
  text_t::const_iterator here = formatstring.begin();
  text_t::const_iterator end = formatstring.end();

  while (here != end) {

    if (*here == '\\')
      text.push_back (*(++here));

    else if (*here == '{') {
      if (!text.empty()) {
	formatlistptr->command = comText;
	formatlistptr->text = text;
	formatlistptr->nextptr = new format_t();
	formatlistptr = formatlistptr->nextptr;
	
	text.clear();
      }
      if (parse_action (++here, end, formatlistptr, metadata, 
			getParents, metamap, metacount)) {
	formatlistptr->nextptr = new format_t();
	formatlistptr = formatlistptr->nextptr;
	if (here == end) break;
      }
    } else if (*here == '[') {
      if (!text.empty()) {
	formatlistptr->command = comText;
	formatlistptr->text = text;
	formatlistptr->nextptr = new format_t();
	formatlistptr = formatlistptr->nextptr;

	text.clear();
      }
      text_t meta;
      here ++;
      while (*here != ']') {
	if (here == end) return false;
	meta.push_back (*here);
	here ++;
      }
      if (parse_meta (meta, metacount, formatlistptr, metadata, getParents, metamap)) {
	formatlistptr->nextptr = new format_t();
	formatlistptr = formatlistptr->nextptr;
      }

    } else 
      text.push_back (*here);

    here ++;
  }
  if (!text.empty()) {
    formatlistptr->command = comText;
    formatlistptr->text = text;
    formatlistptr->nextptr = new format_t();
    formatlistptr = formatlistptr->nextptr;

  }
  return true;
}


static bool parse_action (text_t::const_iterator &here, const text_t::const_iterator &end, 
			  format_t *formatlistptr, text_tarray &metadata, bool &getParents,
			  text_tmap &metamap, int &metacount) {

  text_t::const_iterator it = findchar (here, end, '}');
  if (it == end) return false;

  text_t com = substr (here, it);
  here = findchar (it, end, '{');
  if (here == end) return false;
  else here ++;

  if (com == "If") formatlistptr->command = comIf;
  else if (com == "Or") formatlistptr->command = comOr;
  else return false;

  int curlycount = 0;
  int commacount = 0;
  text_t text;
  while (here != end) {
    if (*here == '{') {curlycount ++; text.push_back(*here);}
    else if (*here == '}' && curlycount > 0) {
      curlycount --; 
      text.push_back(*here);
    }

    else if ((*here == ',' || *here == '}') && curlycount <= 0) {

      if (formatlistptr->command == comOr) {
	// the {Or}{this, or this, or this, or this} statement
	// or'ed statements may be either [metadata] or plain text
	format_t *or_ptr;
	
	// find the next unused orptr
	if (formatlistptr->orptr == NULL) {
	  formatlistptr->orptr = new format_t();
	  or_ptr = formatlistptr->orptr;
	} else {
	  or_ptr = formatlistptr->orptr;
	  while (or_ptr->nextptr != NULL)
	    or_ptr = or_ptr->nextptr;
	  or_ptr->nextptr = new format_t();
	  or_ptr = or_ptr->nextptr;
	}

	text_t::const_iterator beginbracket = text.begin();
	text_t::const_iterator endbracket = (text.end() - 1);
	if ((*beginbracket == '[') && (*endbracket == ']')) {
	  // it's metadata
	  text_t meta = substr (beginbracket+1, endbracket);
	  parse_meta (meta, metacount, or_ptr, metadata, getParents, metamap);

	} else {
	  // assume it's plain text
	  or_ptr->command = comText;
	  or_ptr->text = text;
	}
	text.clear();

      } else {
	// the {If}{decide,do,else} statement
	if (commacount == 0) {
	  // If decision only supports metadata at present
	  
	  // remove the surrounding square brackets
	  text_t::const_iterator beginbracket = text.begin();
	  text_t::const_iterator endbracket = (text.end() - 1);
	  if ((*beginbracket == '[') && (*endbracket == ']')) {
	    text_t meta = substr (beginbracket+1, endbracket);
	    decision_t decision;
	    parse_meta (meta, metacount, formatlistptr->decision, 
			metadata, getParents, metamap);
	    commacount ++;
	    text.clear();
	  } 
	  
	} else if (commacount == 1) {
	  formatlistptr->ifptr = new format_t();
	  parse_string (text, formatlistptr->ifptr, metadata, 
			getParents, metamap, metacount);
	  commacount ++;
	  text.clear();
	  
	} else if (commacount == 2) {
	  formatlistptr->elseptr = new format_t();
	  parse_string (text, formatlistptr->elseptr, metadata, 
			getParents, metamap, metacount);
	  commacount ++;
	  text.clear();
	}
      }
      if (*here == '}') break;
    }

    else 
      text.push_back(*here);
    
    here ++;
  }

  return true;
}


bool parse_formatstring (const text_t &formatstring, format_t *formatlistptr, 
			 text_tarray &metadata, bool &getParents) {

  formatlistptr->clear();
  metadata.erase (metadata.begin(), metadata.end());
  getParents = false;

  text_tmap metamap;
  int metacount = 0;
  return (parse_string (formatstring, formatlistptr, metadata, 
			getParents, metamap, metacount));
}


// note: all the format_date stuff is assuming that all Date metadata is going to
// be of the form yyyymmdd, this is of course, crap ;)

static text_t get_meta (const ResultDocInfo_t &docinfo, const metadata_t &meta) {
  int metasize = docinfo.metadata.size();
  int mindex = meta.metaindex;
  if (metasize < 1 || metasize <= mindex) return "";
  int valuesize = docinfo.metadata[mindex].values.size();

  switch (meta.parentcommand) {
  case pNone:
    if (meta.metaname == "Date") 
      return format_date (docinfo.metadata[mindex].values.back());
    return docinfo.metadata[mindex].values.back();

  case pImmediate:
    if (valuesize > 1) {
      if (meta.metaname == "Date") 
	return format_date (docinfo.metadata[mindex].values[metasize-2]);
      return docinfo.metadata[mindex].values[metasize-2];
    }
    break;

  case pTop:
    if (valuesize > 1) {
      if (meta.metaname == "Date") 
	return format_date (docinfo.metadata[mindex].values[0]);
      return docinfo.metadata[mindex].values[0];
    }
    break;

  case pIndex:
    if (valuesize > meta.parentindex) {
      if (meta.metaname == "Date") 
	return format_date (docinfo.metadata[mindex].values[meta.parentindex]);
      return docinfo.metadata[mindex].values[meta.parentindex];
    }
    break;

  case pAll:
    bool first = true;
    text_t tmp;
    if (valuesize > 1) {
      text_tarray::const_iterator here = docinfo.metadata[mindex].values.begin();
      // don't want last value as that's the value of the current level (i.e. it's
      // not a parent
      text_tarray::const_iterator end = docinfo.metadata[mindex].values.end() - 1;
      while (here != end) {
	if (!first) tmp += meta.parentoptions;
	if (meta.metaname == "Date") tmp += format_date (*here);
	else tmp += *here;
	first = false;
	here ++;
      }
    }
    return tmp;
  }
  return "";
}

static text_t get_or (const ResultDocInfo_t &docinfo, format_t *orptr,
		      const text_t &link, const text_t &icon, const text_t &text) {

  text_t tmp;
  while (orptr != NULL) {

    tmp = format_string (docinfo, orptr, link, icon, text);
    if (!tmp.empty()) return tmp;

    orptr = orptr->nextptr;
  }
  return "";
}

static text_t get_if (const ResultDocInfo_t &docinfo, const decision_t &decision,
		      format_t *ifptr, format_t *elseptr, const text_t &link, 
		      const text_t &icon, const text_t &text) {

  // not much of a choice yet ...
  if (decision.command == dMeta) {
    if (get_meta (docinfo, decision.meta) != "") {
      if (ifptr != NULL)
	return get_formatted_string (docinfo, ifptr, link, icon, text);
    }
    else {
      if (elseptr != NULL)
	return get_formatted_string (docinfo, elseptr, link, icon, text);
    }
  }
  return "";
}

text_t format_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
		      const text_t &link, const text_t &icon, const text_t &text) {

  if (formatlistptr == NULL) return "";

  switch (formatlistptr->command) {
  case comText:
    return formatlistptr->text;
  case comLink:
    return link;
  case comEndLink:
    if (link.empty) return "";
    else return "</a>";
  case comIcon:
    return icon;
  case comNum:
    return docinfo.result_num;
  case comMeta:
    return get_meta (docinfo, formatlistptr->meta);
  case comDoc:
    return text;
  case comIf:
    return get_if (docinfo, formatlistptr->decision, formatlistptr->ifptr, 
		   formatlistptr->elseptr, link, icon, text);
  case comOr:
    return get_or (docinfo, formatlistptr->orptr, link, icon, text);
  }
  return "";
}


text_t get_formatted_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
			     const text_t &link, const text_t &icon) {

  text_t text;

  text_t ft;
  while (formatlistptr != NULL) {
    ft += format_string (docinfo, formatlistptr, link, icon, text);
    formatlistptr = formatlistptr->nextptr;
  }
  return ft;
}


text_t get_formatted_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr) {

  text_t link = "<a href=\"_httpdocument_&cl=search&d=" + docinfo.OID + "\">";
  text_t icon = "_icontext_";
  text_t text;

  text_t ft;
  while (formatlistptr != NULL) {
    ft += format_string (docinfo, formatlistptr, link, icon, text);
    formatlistptr = formatlistptr->nextptr;
  }
  return ft;
}


text_t get_formatted_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
			     const text_t &text) {

  text_t link = "<a href=\"_httpdocument_&cl=search&d=" + docinfo.OID + "\">";
  text_t icon = "_icontext_";

  text_t ft;
  while (formatlistptr != NULL) {
    ft += format_string (docinfo, formatlistptr, link, icon, text);
    formatlistptr = formatlistptr->nextptr;
  }
  return ft;
}


text_t get_formatted_string (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
			     const text_t &link, const text_t &icon, const text_t &text) {

  text_t ft;
  while (formatlistptr != NULL) {
    ft += format_string (docinfo, formatlistptr, link, icon, text);
    formatlistptr = formatlistptr->nextptr;
  }
  return ft;
}



